<?php

require(dirname(__FILE__) . '/sdk/sdk_autoload.php');

use B2P\Client;
use B2P\Responses\Error as SDK_Error;
use B2P\Models\Orders\Order as SDK_Order;
use B2P\Responses\Operation as SDK_Operation;
use B2P\Models\Interfaces\CreditOrder;
use B2P\Models\Orders\InstallmentsOrder;
use B2P\Models\Enums\CurrencyCode;

/**
 * @package waPlugins
 * @subpackage Payment
 * @name Paygine
 * @description Paygine payment module
 * @payment-type online
 *
 * @property-read string $sector_id
 * @property-read string $password
 * @property-read string $payment_method
 * @property-read string $modal_payform
 * @property-read string $test_mode
 * @property-read string $tax
 * @property-read string $hash_algo
 * @property-read int $notify_customer
 *
 * tested on Webasyst 2.9.4 \ Shop-Script 10.0.2
 */
class payginePayment extends waPayment implements waIPayment, waIPaymentCapture, waIPaymentRefund, waIPaymentCancel
{
	const REFERENCE_PATTERN = '/^([a-z]+)_(.+)_(.+)$/';
	const PG_OPERATION_PURCHASE = 'PURCHASE';
	const PG_OPERATION_AUTHORIZE = 'AUTHORIZE';
	const PG_OPERATION_PURCHASE_BY_QR = 'PURCHASE_BY_QR';
	const PG_OPERATION_COMPLETE = 'COMPLETE';
	const PG_OPERATION_LOAN = 'LOAN';
	const PG_OPERATION_REFUND = 'REVERSE';
	const OPERATION_APPROVED = 'APPROVED';
	const ORDER_REGISTERED = 'REGISTERED';
	const ORDER_AUTHORIZED = 'AUTHORIZED';
	const ORDER_COMPLETED = 'COMPLETED';
	const ORDER_CANCELED = 'CANCELED';
	const ORDER_BLOCKED = 'BLOCKED';
	const ORDER_EXPIRED = 'EXPIRED';
	const FP_NAME_MAX_LENGTH = 128;
	const PM_PURCHASE = 'purchase';
	const PM_AUTHORIZE = 'authorize';
	const PM_PWI = 'purchaseWithInstallment';
	const PM_AWI = 'authorizeWithInstallment';
	const PM_SBP = 'purchaseSBP';
	const PM_LOAN = 'loan';
	const WA_ORDER_NEW = 'new';
	const WA_ORDER_PAID = 'paid';
	const WA_ORDER_AUTH = 'auth';
	const WA_ORDER_REFUNDED = 'refunded';
	private $client;
	private $order_id;
	private $order_paid = 0;
	private $callback_request = array();
	
	/**
	 * @return array
	 * @see waPayment::supportedOperations()
	 */
	public function supportedOperations() {
		return array(
			self::OPERATION_CAPTURE,
			self::OPERATION_AUTH_CAPTURE,
			self::OPERATION_REFUND,
		);
	}
	
	public function allowedCurrency() {
		return 'RUB';
	}
	
	protected function getClient(): void {
		if(isset($this->client)) return;
		$test_mode = ($this->test_mode === '1');
		$hash_algo = ($this->hash_algo === 'sha256');
		if (empty((int) $this->sector_id) || empty($this->password))
			throw new waException('Sector ID or Password undefined');
		$this->client = new Client((int) $this->sector_id, $this->password, $test_mode, $hash_algo);
	}
	
	public function payment($payment_form_data, $order_data, $auto_submit = false) {
		$order_id = filter_var($order_data['id'], FILTER_SANITIZE_NUMBER_INT);
		$order_params_model = new shopOrderParamsModel();
		$pc_order_id = (int) $order_params_model->getOne($order_id, 'native_id');
		// There is no payment url for "notify customer" orders
		$order_notify_customer = $order_params_model->getOne($order_id, 'notify_customer');
		$state = '<h3><strong>' . $this->_w('The order is awaiting confirmation by the manager') . '</strong></h3>';
		if($order_notify_customer && !$pc_order_id)
			return $state;
		if(!$order_notify_customer && $this->notify_customer){
			$order_params_model->setOne($order_id, 'notify_customer', 1);
			return $state;
		}
		
		$order = waOrder::factory($order_data);
		
		if(!$pc_order_id){
			$pc_order_id = $this->register_order($order, $payment_form_data);
		}
		
		$data = [
			'sector' => $this->sector_id,
			'id' => $pc_order_id,
		];
		
		$shop_cart = $this->calc_shop_cart($order);
		if(($this->payment_method === self::PM_PWI || $this->payment_method === self::PM_AWI) && $shop_cart)
			$data['shop_cart'] = base64_encode(json_encode($shop_cart, JSON_UNESCAPED_UNICODE));
		
		if($this->payment_method === self::PM_LOAN)
			$data['reference'] = $this->createReference($order->id);
		
		$this->getClient();
		$payment_url = call_user_func([$this->client, $this->payment_method], $data);
		$view = wa()->getView();
		$configManager = $this->client->getConfigManager();
		$view->assign(
			array(
				'base_url' => $configManager->getUrl(),
				'form_url' => $payment_url,
				'auto_submit' => $auto_submit,
				'args' => [],
				'plugin' => $this,
				'modal_url' => $payment_url
			)
		);
		
		return $view->fetch($this->path . '/templates/payment.html');
	}
	
	public function register_order($order, $payment_form_data = [], $notify_customer = 0) : int
	{
		$order_params_model = new shopOrderParamsModel();
		$email = htmlspecialchars($this->getCustomerEmail($payment_form_data, $order), ENT_QUOTES);
		$tax = (isset($this->tax) && $this->tax > 0 && $this->tax <= 6) ? (int) $this->tax : 6;
		$this->getClient();
		$currency = CurrencyCode::cases()[$order['currency']];
		if(!$currency)
			return $this->_w('Unable to use selected payment method with this currency');
		
		$order_amount = $this->centify($order['amount']);
		
		$data = array(
			'sector' => $this->sector_id,
			'amount' => $order_amount,
			'currency' => $currency,
			'description' => '#' . $order->id,
			'reference' => $this->createReference($order->id),
			'fiscal_positions' => $this->calc_fiscal_positions($order, $tax),
			'email' => $email,
			'mode' => 0,
			'url' => $this->getRelayUrl(),
			'notify_customer' => $notify_customer,
		);
		
		try {
			$pc_order = $this->client->register($data);
			if($pc_order instanceof SDK_Error)
				throw new waException($this->_w('Order registration failed.') . ' ' . $pc_order->description);
			$pc_order_id = (int) $pc_order->id;
			if(!$pc_order_id)
				throw new waException($this->_w('Failed to get Paygine order ID'));
		} catch (waException $e) {
			self::log($this->id, $e->getMessage());
			return $e->getMessage();
		}
		$order_params_model->setOne($order->id, 'native_id', $pc_order_id);
		$order_params_model->setOne($order->id, 'pc_order_state', $pc_order->getState());
		
		$shop_state_id = $this->getSettings('order_registered') ?: self::WA_ORDER_NEW;
		$order_model = new shopOrderModel();
		$order_model->updateById($order->id, array('state_id' => $shop_state_id));
		if($notify_customer)
			$this->addOrderLog($order->id, $this->_w('A link to pay for the order has been sent to email') . ' <strong>' . $email . '</strong>');
		return $pc_order_id;
	}
	
	/**
	 * @param $order waOrder
	 * @param $tax int
	 * @return array
	 */
	private function calc_fiscal_positions(waOrder $order, int $tax): array {
		$fiscal_positions = [];
		$fiscal_amount = 0;
		$order_amount = $this->centify($order['amount']);
		foreach ($order->items as $key => $item) {
			$fiscal_positions[$key]['quantity'] = $item['quantity'];
			$item_price = round($item['price'], 2) - round(ifset($item['discount'], 0.0), 2);
			$element_price = $this->centify($item_price);
			$fiscal_positions[$key]['amount'] = $element_price;
			$fiscal_positions[$key]['tax'] = $tax;
			$fiscal_positions[$key]['name'] = str_replace([';', '|'], '', mb_substr($item['name'], 0, self::FP_NAME_MAX_LENGTH));
			$fiscal_amount += $item['quantity'] * $element_price;
		}
		
		if ($order->shipping > 0) {
			$shipping_price = round($order->shipping, 2);
			$fiscal_element_price = $this->centify($shipping_price);
			$fiscal_positions[] = [
				'quantity' => 1,
				'amount' => $fiscal_element_price,
				'tax' => $tax,
				'name' => $this->_w('Delivery')
			];
			$fiscal_amount += $fiscal_element_price;
		}
		
		$fiscalDiff = abs($fiscal_amount - $order_amount);
		if ($fiscalDiff)
			$fiscal_positions[] = [1, $fiscalDiff, $tax, $this->_w('Discount'), 14];
		return $fiscal_positions;
	}
	
	/**
	 * @param $order waOrder
	 * @return array
	 */
	private function calc_shop_cart(waOrder $order): array {
		$shop_cart = [];
		$fiscal_amount = 0;
		$order_amount = $this->centify($order['amount']);
		foreach ($order->items as $item) {
			$item_price = round($item['price'], 2) - round(ifset($item['discount'], 0.0), 2);
			$element_price = $this->centify($item_price);
			$fiscal_amount += $item['quantity'] * $element_price;
			$shop_cart[] = [
				'quantityGoods' => (int) $item['quantity'],
				'goodCost' => $item_price,
				'name' => $item['name']
			];
		}
		if ($order->shipping > 0) {
			$shipping_price = round($order->shipping, 2);
			$fiscal_element_price = $this->centify($shipping_price);
			$shop_cart[] = [
				'quantityGoods' => 1,
				'goodCost' => $shipping_price,
				'name' => $this->_w('Delivery'),
			];
			$fiscal_amount += $fiscal_element_price;
		}
		return abs($fiscal_amount - $order_amount) ? [] : $shop_cart;
	}
	
	/**
	 * @param $payment_form_data array
	 * @param $order waOrder
	 * @return string
	 */
	private function getCustomerEmail($payment_form_data, $order) {
		if (isset($payment_form_data['customer_email']))
			return $payment_form_data['customer_email'];
		if (isset($order['customer_info']['email']))
			return $order['customer_info']['email'];
		if (isset($order['contact_email']))
			return $order['contact_email'];
		
		$contact = new waContact($order->contact_id);
		return isset($contact->get('email', 'value')[0]) ? $contact->get('email', 'value')[0] : '';
	}
	
	protected function callbackInit($request) {
		try {
			if (isset($request['callback']) && $request['callback'] == 1) {
				$response = file_get_contents('php://input');
				$pc_operation = $this->parseXML($response);
				$this->parseReference($pc_operation['reference']);
				$this->callback_request = $pc_operation;
			} else {
				$this->parseReference($request['reference']);
			}
		} catch (waPaymentException $e) {
			self::log($this->id, $e->getMessage());
			return $e->getMessage();
		}
		
		return parent::callbackInit($request);
	}
	
	protected function callbackHandler($request) {
		if(!empty($request['error'])) {
			$redirect = wa()->getRouteUrl('shop/frontend/myOrder', ['id' => $this->order_id]);
			if($this->modal_payform)
				$this->JsRedirect($redirect);
			wa()->getResponse()->redirect($redirect);
		}
		
		$order_model = new shopOrderModel();
		$order_data = $order_model->getById($this->order_id);
		if(!$order_data)
			throw new waException('Order not found');
		$order_params_model = new shopOrderParamsModel();
		$this->order_paid = $order_params_model->getOne($this->order_id, 'paid');
		// if request from redirect and order already paid
		if(!$this->callback_request && $this->order_paid){
			$redirect = $this->getAdapter()->getBackUrl();
			return ['redirect' => $redirect];
		}
		
		
		$this->getClient();
		// getting order from request body in notify case or from PC in redirect case
		$pc_order_id = $this->callback_request ? $this->callback_request['order_id'] : (int) $request['id'];
		$pc_order = $this->client->order(['id' => $pc_order_id]);
		if (!($pc_order instanceof SDK_Order))
			throw new waException('Failed to get Order entity from paysystem');
		// getting operation from request body in notify case or from PC in redirect case
		$pc_operation_id = $this->callback_request ? $this->callback_request['id'] : (int) $request['operation'];
		$pc_operation = $pc_order->getOperation($pc_operation_id);
		if (!($pc_operation instanceof SDK_Operation))
			throw new waException('Failed to get Operation entity from paysystem');
		
		$transaction_data = parent::formalizeData($request);
		$transaction_data['id'] = $this->id;
		$transaction_data['order_id'] = $this->order_id;
		$transaction_data['native_id'] = $pc_order->id;
		$transaction_data['currency_id'] = $order_data['currency'];
		
		$is_loan = $pc_order instanceof CreditOrder;
		$payment_method = $this->getPcPaymentMethod($pc_operation, $pc_order);
		
		if ($this->callback_request) {
			if((string) $pc_operation->state !== self::OPERATION_APPROVED)
				exit('ok');
			
			// Ignoring COMPLETE operation for loan orders (complete on LOAN operation)
			if($is_loan && (string) $pc_operation->type === self::PG_OPERATION_COMPLETE){
				$this->addOrderLog($order_data['id'], 'CALLBACK : Order confirmed');
				exit('ok');
			}
			$transaction_data['amount'] = round($pc_order->amount / 100, 2);
			$app_payment_method = $this->gatewayNotify($pc_operation, $order_params_model, $payment_method, $is_loan, $transaction_data);
			
			$transaction_data = $this->saveTransaction($transaction_data, $request);
			$this->execAppCallback($app_payment_method, $transaction_data);
			
			$order_model->updateById($this->order_id, ['state_id' => $this->getCustomOrderStatus((string) $pc_operation->type, $pc_order->getState(), $is_loan)]);
			
			exit('ok');
		}
		
		if ((string) $pc_operation->state === self::OPERATION_APPROVED) {
			$status = waAppPayment::URL_SUCCESS;
			$callback = ($pc_order->getState() == self::ORDER_AUTHORIZED) ? self::CALLBACK_AUTH : self::CALLBACK_PAYMENT;
			$transaction_data['state'] = ($pc_order->getState() == self::ORDER_AUTHORIZED) ? self::STATE_AUTH : self::STATE_CAPTURED;
		} else {
			$status = waAppPayment::URL_FAIL;
			$callback = self::CALLBACK_DECLINE;
			$transaction_data['state'] = self::STATE_DECLINED;
			$transaction_data['error'] = true;
			$this->addOrderLog($order_data['id'], 'Operation not approved (' . (string) $pc_operation->state . ')');
		}
		
		$payment_methods = $this->getAllPaymentMethods(false);
		
		$this->addOrderLog($order_data['id'], $this->_w('Payment method') . ': '. $payment_methods[$payment_method]);
		
		$transaction_data['type'] = $pc_order->getState() == self::ORDER_AUTHORIZED ? self::OPERATION_AUTH_ONLY : self::OPERATION_CAPTURE;
		$transaction_data['amount'] = $order_data['total'];
		$transaction_data = $this->saveTransaction($transaction_data, $request);
		$this->execAppCallback($callback, $transaction_data);
		
		$order_params_model->setOne($this->order_id, 'payment_method', $payment_method);
		$order_params_model->setOne($this->order_id, 'paid', 1);
		$order_params_model->setOne($this->order_id, 'pc_order_state', $pc_order->getState());
		// rewrite the order status for a custom order in case of payment for the order
		if ((string) $pc_operation->state === self::OPERATION_APPROVED)
			$order_model->updateById($this->order_id, ['state_id' => $this->getCustomOrderStatus((string) $pc_operation->type, $pc_order->getState(), $is_loan)]);
		
		$redirect = $this->getAdapter()->getBackUrl($status, $transaction_data);
		
		if($this->modal_payform)
			$this->JsRedirect($redirect);
		
		return array(
			'redirect' => $redirect
		);
	}
	
	private function gatewayNotify($pc_operation, $order_params_model, $payment_method, $is_loan, &$transaction_data) {
		header('Content-type: text/plain');
		
		$app_payment_method = null;
		switch((string) $pc_operation->type){
			case self::PG_OPERATION_PURCHASE:
			case self::PG_OPERATION_PURCHASE_BY_QR:
				// The order has already been paid
				if($this->order_paid) exit('ok');
				$app_payment_method = self::CALLBACK_PAYMENT;
				$transaction_data['type'] = self::OPERATION_CAPTURE;
				$transaction_data['state'] = self::STATE_CAPTURED;
				$order_params_model->setOne($this->order_id, 'payment_method', $payment_method);
				$order_params_model->setOne($this->order_id, 'paid', 1);
				break;
			case self::PG_OPERATION_AUTHORIZE:
				// The order has already been authorized
				if($this->order_paid) exit('ok');
				$app_payment_method = self::CALLBACK_AUTH;
				$transaction_data['type'] = self::OPERATION_AUTH_ONLY;
				$transaction_data['state'] = self::STATE_AUTH;
				$order_params_model->setOne($this->order_id, 'payment_method', $payment_method);
				$order_params_model->setOne($this->order_id, 'paid', 1);
				break;
			case self::PG_OPERATION_COMPLETE:
				// Operation COMPLETE works for non-loan orders only
				if($is_loan) break;
				$app_payment_method = self::CALLBACK_PAYMENT;
				$transaction_data['type'] = self::OPERATION_CAPTURE;
				$transaction_data['state'] = self::STATE_CAPTURED;
				break;
			case self::PG_OPERATION_LOAN:
				// Operation LOAN works for loan COMPLETED orders only
				if(!$is_loan || (string) $pc_operation->order_state !== self::ORDER_COMPLETED) break;
				$app_payment_method = self::CALLBACK_PAYMENT;
				$transaction_data['type'] = self::OPERATION_CAPTURE;
				$transaction_data['state'] = self::STATE_CAPTURED;
				break;
			case self::PG_OPERATION_REFUND:
				$app_payment_method = self::CALLBACK_REFUND;
				$transaction_data['type'] = self::OPERATION_REFUND;
				$transaction_data['state'] = self::STATE_REFUNDED;
				break;
		}
		
		if(!$app_payment_method)
			throw new waPaymentException('Undefined operation type');
		
		return $app_payment_method;
	}
	
	public function parseXML($response) {
		if (!$response)
			throw new waException('Empty response');
		$xml = simplexml_load_string($response);
		if (!$xml)
			throw new waException('Non valid XML was received');
		$valid_xml = json_decode(json_encode($xml), true);
		if (!$valid_xml)
			throw new waException('Non valid XML was received');
		return $valid_xml;
	}
	
	function createReference($order_id) {
		return $this->app_id . '_' . $this->merchant_id . '_' . $order_id;
	}
	
	private function parseReference($reference) {
		if(!$reference or !preg_match(self::REFERENCE_PATTERN, $reference, $match))
			throw new waPaymentException('Order reference is undefined');
		if(empty($match[1]) || empty($match[2]) || empty($match[3]))
			throw new waPaymentException('Invalid invoice number');
		$this->app_id = $match[1];
		$this->merchant_id = $match[2];
		$this->order_id = $match[3];
	}
	
	/**
	 * @param $transaction_raw_data
	 * @return array
	 * @throws waException
	 * @throws waPaymentException
	 */
	public function refund($transaction_raw_data) {
		$transaction_raw_data = $this->getRefundTransactionData($transaction_raw_data);
		$transaction = $transaction_raw_data['transaction'];
		try {
			$this->getClient();
			$pc_order = $this->client->order(['id' => $transaction['native_id']]);
			if(!$pc_order->reverse())
				throw new waException('Failed to refund the order');
		} catch (waException $ex) {
			error_log($ex->getMessage());
			$this->addOrderLog($transaction['order_id'], $ex->getMessage(), 'exclamation-red');
			return [
				'result' => 'error',
				'description' => $ex->getMessage()
			];
		}
		
		$refund_transaction = array(
			'native_id'    => $transaction['native_id'],
			'type'         => self::OPERATION_REFUND,
			'result'       => 1,
			'order_id'     => $transaction['order_id'],
			'customer_id'  => $transaction['customer_id'],
			'amount'       => $transaction['amount'],
			'currency_id'  => $transaction['currency_id'],
			'parent_id'    => $transaction['id'],
			'parent_state' => self::STATE_REFUNDED,
			'state'        => self::STATE_REFUNDED,
		);
		$this->saveTransaction($refund_transaction);
		
		return [
			'result' => 0,
			'amount' => round($pc_order->amount / 100, 2)
		];
	}
	
	/**
	 * @param $transaction_raw_data
	 * @return array
	 * @throws waException
	 */
	public function capture($transaction_raw_data) {
		$transaction = $transaction_raw_data['transaction'];
		try {
			$this->getClient();
			$pc_order = $this->client->order(['id' => $transaction['native_id']]);
			if(!$pc_order->complete())
				throw new waException('Failed to complete the order');
		} catch (waException $ex) {
			error_log($ex->getMessage());
			$this->addOrderLog($transaction['order_id'], $ex->getMessage(), 'exclamation-red');
			return [
				'result' => 'error',
				'description' => $ex->getMessage()
			];
		}
		$order_params_model = new shopOrderParamsModel();
		$payment_method = $order_params_model->getOne($transaction['order_id'], 'payment_method');
		// In the case of "loan", the complete order is executed by the callback of the LOAN APPROVED operation
		if($payment_method === self::PM_LOAN){
			$this->addOrderLog($transaction['order_id'], 'Order confirmed');
			return [];
		}
		$capture_transaction = array(
			'native_id'    => $transaction['native_id'],
			'type'         => self::OPERATION_CAPTURE,
			'result'       => 1,
			'order_id'     => $transaction['order_id'],
			'customer_id'  => $transaction['customer_id'],
			'amount'       => $transaction['amount'],
			'currency_id'  => $transaction['currency_id'],
			'parent_id'    => $transaction['id'],
			'parent_state' => self::STATE_CAPTURED,
			'state'        => self::STATE_CAPTURED,
		);
		$this->saveTransaction($capture_transaction);
		
		return [
			'result' => 0,
			'amount' => round($pc_order->amount / 100, 2)
		];
	}
	
	/**
	 * @param $transaction_raw_data
	 * @return array
	 * @throws waException
	 * @throws waPaymentException
	 */
	public function cancel($transaction_raw_data) {
		return $this->refund($transaction_raw_data);
	}
	
	/**
	 * @return array
	 */
	public function getAllPaymentMethods($asterisk = true) {
		$asterisk = $asterisk ? ' *' : '';
		return array(
			self::PM_PURCHASE => $this->_w('Standard acquiring (one-stage payment)'),
			self::PM_AUTHORIZE => $this->_w('Standard acquiring (two-stage payment)') . $asterisk,
			self::PM_PWI => $this->_w('Plait (one-stage payment)'),
			self::PM_AWI => $this->_w('Plait (two-stage payment)') . $asterisk,
			self::PM_SBP => $this->_w('Fast Payment System'),
			self::PM_LOAN => $this->_w('Loan')
		);
	}
	
	/**
	 * @return array
	 */
	public static function getAllOrderStates() {
		static $res;
		if($res)
			return $res;
		$workflow = new shopWorkflow();
		$res = array(array('value' => '', 'title' => ''));
		foreach ($workflow->getAvailableStates() as $state_id => $state) {
			$res[] = [
				'value' => $state_id,
				'title' => $state['name']
			];
		}
		return $res;
	}

	function addOrderLog($order_id, $text, $icon = '', $action_id = '') {
		$order_log_model = new shopOrderLogModel();
		$icon = $icon ? "<i class=\"icon16 $icon\"></i>" : '';
		$log = array(
			'order_id'        => $order_id,
			'contact_id'      => wa()->getUser()->getId(),
			'action_id'       => $action_id,
			'text'            => $icon . $text,
			'before_state_id' => '',
			'after_state_id'  => '',
		);
		$order_log_model->add($log);
	}
	
	function JsRedirect($url) {
		echo "<script>window.top.location.href = '{$url}'</script>";
		die();
	}
	
	public function centify($value): int {
		return intval(strval($value * 100));
	}
	
	public function getPcPaymentMethod($pc_operation, $pc_order) {
		switch((string) $pc_operation->type) {
			case self::PG_OPERATION_PURCHASE:
				return ($pc_order instanceof InstallmentsOrder) ? self::PM_PWI : self::PM_PURCHASE;
			case self::PG_OPERATION_AUTHORIZE:
				if($pc_order instanceof InstallmentsOrder)
					return self::PM_AWI;
				elseif($pc_order instanceof CreditOrder)
					return self::PM_LOAN;
				else
					return self::PM_AUTHORIZE;
			case self::PG_OPERATION_PURCHASE_BY_QR:
				return self::PM_SBP;
		}
		return '';
	}
	
	public function getCustomOrderStatus($operation_type, $order_state, $is_loan) {
		switch($operation_type){
			case self::PG_OPERATION_AUTHORIZE:
				if($order_state !== self::ORDER_AUTHORIZED) break;
				if($is_loan)
					return !empty($this->getSettings('order_loan')) ? $this->getSettings('order_loan') : self::WA_ORDER_AUTH;
				else
					return !empty($this->getSettings('order_authorized')) ? $this->getSettings('order_authorized') : self::WA_ORDER_AUTH;
			case self::PG_OPERATION_PURCHASE:
			case self::PG_OPERATION_PURCHASE_BY_QR:
			case self::PG_OPERATION_COMPLETE:
			case self::PG_OPERATION_LOAN:
				if($order_state !== self::ORDER_COMPLETED) break;
				// complete loan orders only by LOAN operation(not COMPLETE)
				if($operation_type === self::PG_OPERATION_COMPLETE && $is_loan) break;
				return !empty($this->getSettings('order_completed')) ? $this->getSettings('order_completed') : self::WA_ORDER_PAID;
			case self::PG_OPERATION_REFUND:
				if($order_state !== self::ORDER_CANCELED) break;
				return !empty($this->getSettings('order_canceled')) ? $this->getSettings('order_canceled') : self::WA_ORDER_REFUNDED;
		}
		return false;
	}
	
	public static function TitleCustomControl($name, $params) {
		return '<h4 style="margin: 1em 0 0">' . $params['value'] . '</h4>';
	}
	
	public static function ModalPayformCustomControl($name, $params) {
		$plugin = $params['instance'];
		$view = wa()->getView();
		$view->assign(
			array(
				'name' => $name,
				'id' => $params['id'],
				'value' => $params['value'],
				'plugin' => $plugin,
			)
		);
		return $view->fetch($plugin->path . '/templates/modal_payform.html');
	}
	
	public static function NotifyCustomerCustomControl($name, $params) {
		$plugin = $params['instance'];
		$view = wa()->getView();
		$view->assign(
			array(
				'name' => $name,
				'id' => $params['id'],
				'value' => $params['value'],
				'plugin' => $plugin,
				'shop_plugin_enabled' => self::shopPluginEnabled(),
			)
		);
		return $view->fetch($plugin->path . '/templates/notify_customer.html');
	}
	
	public static function shopPluginEnabled(){
		$shop_plugins = wa('shop')->getConfig()->getPlugins();
		if(empty($shop_plugins['paygineconfirm'])) return false;
		return true;
	}
	
	public function saveSettings($settings = array()){
		$wf_config = shopWorkflow::getConfig();
		if(ifset($settings['notify_customer']) === '1'){
			$wf_config['actions']['process']['classname'] = 'shopPaygineConfirmPluginProcessAction';
			shopWorkflow::setConfig($wf_config);
		}elseif($wf_config['actions']['process']['classname'] !== 'shopWorkflowProcessAction'){
			$wf_config['actions']['process']['classname'] = 'shopWorkflowProcessAction';
			shopWorkflow::setConfig($wf_config);
		}
		parent::saveSettings($settings);
	}
}